/**
 * Primer veoma jednostavne rekurzivne funkcije u kojoj se pozivi linearno
 * spuštaju do trivijalnog uslova i potom se direktno vraćaju do originalnog
 * poziva. Pri tome ne postoje ni nikakve dodatne promenljive ili pozivi.
 */
public class Faktorijel {

	/**
	 * Originalna funkcija koja će biti simulirana
	 */
	static int faktorijelR(int n) {
		if (n <= 1)
			return 1;
		else
			return n * faktorijelR(n - 1);
	}

	/**
	 * Simulirana verzija rekurzivne funkcije, u kojoj se samo posmatra
	 * rekurzivni spust i povratak, posto u ovoj jednostavnoj funkciji i ne
	 * postoji nista drugo. Takodje se na steku samo cuva vrednost parametra
	 * 'n', nije pravljen infotip jer nema potrebe.
	 */
	static int faktorijelS(int n) {
		Stek<Integer> s = new Stek<Integer>(100);

		// rekurzivni pozivi do trivijalnog slučaja
		while (n > 1) {
			s.stavi(n);
			n--;
		}

		// trivijalni slučaj
		int rez = 1;

		// povratak iz rekurzije
		while (!s.jePrazan()) {
			n = s.skiniVrh();
			rez = rez * n;
		}
		return rez;
	}

	// eksplicitni infotip koji se uklapa u algoritam pretvaranja funkcije
	static private class InfoTip {
		int n;
		int adr; // nema potrebe kad ima samo jedan poziv
	}

	/**
	 * Simulirana verzija rekurzivne funkcije, sa svim detaljima datog algoritma
	 * za transformaciju. Mnogi detalji su nepotrebni za ovu funkciju, sto je i
	 * oznaceno, medjutim nije greska navesti ih 
	 */
	static int faktorijelSDetaljno(int n) {
		Stek<InfoTip> s = new Stek<>(100);
		int rez;
		InfoTip el;

		do {
			// rekurzivni pozivi do trivijalnog slučaja
			while (n > 1) {
				el = new InfoTip();
				el.n = n;
				el.adr = 1;// drugo ni ne postoji
				s.stavi(el);
				n--;
			}

			// trivijalni slučaj
			rez = 1;

			// povratak iz rekurzije
			boolean nemaPoziva = true;
			while (!s.jePrazan() && nemaPoziva) {
				el = s.skiniVrh();
				n = el.n;
				if (el.adr == 1) {
					rez = rez * n;
				} // ne postoje druge adrese
			}

			// posto se 'nemaPoziva' ne menja
			// u ovom momentu ce s uvek biti prazan
			// pa nema potrebe ni za ovom petljom
		} while (!s.jePrazan());

		return rez;
	}

	public static void main(String[] args) {
		int n = Svetovid.in.readInt("n = ?");
		System.out.print("n! = ");
		System.out.print(faktorijelR(n));
		System.out.print(" = ");
		System.out.print(faktorijelS(n));
		System.out.print(" = ");
		System.out.println(faktorijelSDetaljno(n));
	}
}