/**
 * Medjuspremnik (a.k.a. buffer) sluzi sa skladistenje podataka pre nego sto se
 * proslede dalje.
 * 
 * U ovom primeru se pokazuje kako se podaci prvo prikupljaju pre nego sto se
 * zapravo ispisu, cime se dobija na efikasnosti ispisa, posto je tipicno
 * operacija rada sa nekim spoljim uredjajem relativno spora u odnosu na interne
 * operacije, a sa druge strane su takve operacije tipicno slicne brzine i kad
 * se salje vise podataka odjednom.
 * 
 * Naravno ovo je samo ilustracija, operacije ispisa i na ekran i na disk su
 * tipicno vec optimizovane medjuspremnicima.
 *
 */
public class MedjuSpremnik {

	public static final int VEL_SPREMNIKA = 33;

	public static Red<Character> spremnik = new Red<>(VEL_SPREMNIKA);

	public static String izlaznifajl = "izlaz.txt";

	public static void pisiZnak(char c) {
		if (spremnik.jePun()) {
			isprazni();
		}
		spremnik.naKraj(c);
	}

	public static void isprazni() {
		while (!spremnik.jePrazan()) {
			char c = spremnik.izbaciPrvi();
			Svetovid.out(izlaznifajl).print(c);
		}

	}

	public static void pisi(String str) {
		for (int i = 0; i < str.length(); i++) {
			pisiZnak(str.charAt(i));
		}
	}

	public static void main(String[] args) {
		System.out.println("Unesite reci koje treba ispisati u fajl:");
		String str = Svetovid.in.readLine("rec?");
		while (!str.equals(".")) {
			pisi(str);
			str = Svetovid.in.readLine("rec?");
		}
		Svetovid.out(izlaznifajl).close();
	}

}
